/*
 * Copyright (C) 2013-2024  Eugen Wissner <belka@caraus.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "timedate.h"

bool timeout_callback(Glib::RefPtr<Glib::MainLoop> loop2quit)
{
    loop2quit->quit();
    return false;
}

constexpr unsigned int default_exit_sec = 300;

int main(int argc, char **argv)
{
    Gio::init();
    auto timedate1 = std::make_unique<dlackware::timedate::timedate1>();

    guint owner_id = Gio::DBus::own_name(Gio::DBus::BUS_TYPE_SYSTEM,
            Glib::ustring(dlackware::timedate::bus_name.cbegin(), dlackware::timedate::bus_name.cend()),
            std::bind(&dlackware::timedate::timedate1::on_bus_acquired, timedate1.get(),
                std::placeholders::_1, std::placeholders::_2),
            Gio::DBus::SlotNameAcquired(),
            std::bind(&dlackware::timedate::timedate1::on_name_lost, timedate1.get(),
                std::placeholders::_1, std::placeholders::_2),
            Gio::DBus::BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT | Gio::DBus::BUS_NAME_OWNER_FLAGS_REPLACE);

    Glib::RefPtr<Glib::MainLoop> loop = Glib::MainLoop::create(false);

    Glib::signal_timeout().connect_seconds(sigc::bind(&timeout_callback, loop), default_exit_sec);
    loop->run();

    Gio::DBus::unown_name(owner_id);

    return EXIT_SUCCESS;
}
