/*
 * Copyright (C) 2013-2024  Eugen Wissner <belka@caraus.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <dbus/dbus.h>
#include <glibmm.h>
#include <giomm.h>
#include "resources.h"

namespace dlackware::timedate
{
    constexpr const char *bus_path = "/org/freedesktop/timedate1";
    constexpr const std::string_view bus_name = "org.freedesktop.timedate1";
    constexpr const std::string_view zoneinfo_database = "/usr/share/zoneinfo";

    // Returns the system time zone.
    Glib::ustring timezone();

    // Returns if the hardware clock is set to local time or not
    bool local_rtc();

    // Returns whether NTP is enabled.
    bool ntp();

    // Returns whether NTP is available.
    bool can_ntp();

    // Shows whether the kernel reports the time as synchronized.
    bool ntp_synchronized();

    // Shows the current time in the RTC.
    std::uint64_t rtc_time_usec();

    // Shows the current time on the system.
    std::uint64_t time_usec();

    // Returns the timezones available on the system.
    std::vector<Glib::ustring> list_timezones();

    // Sets the system time zone to the one passed by the argument
    // Throws std::filesystem::filesystem_error.
    void set_timezone(const Glib::ustring& zone, bool user_interaction);

    // Sets NTP
    // Throws std::filesystem::filesystem_error.
    void set_ntp(bool use_ntp, bool user_interaction);

    // Changes the date/time
    // Takes the amount of seconds since UNIX epoche and 
    // Throws std::system_error.
    void set_time(std::int64_t usec_utc, bool relative, bool user_interaction);

    // Controls whether the RTC is local time or UTC.
    void set_local_rtc(bool local_rtc, bool fix_system, bool user_interaction);

    class timedate1
    {
        const Gio::DBus::InterfaceVTable interface_vtable;

    public:
        timedate1();

        void on_bus_acquired(const Glib::RefPtr<Gio::DBus::Connection>& connection, const Glib::ustring& name);
        void on_name_lost(const Glib::RefPtr<Gio::DBus::Connection>& connection, const Glib::ustring& name);
    };
}
